import typer
import json
import os

from .config.base import ROOT_PATH, TRASH_PATH, CONFIG_PATH
from .common.file import create_file

def create_root():
    if not os.path.exists(ROOT_PATH):
        os.mkdir(ROOT_PATH)
    if not os.path.exists(TRASH_PATH):
        os.mkdir(TRASH_PATH)
    if not os.path.exists(CONFIG_PATH):
        config_template = {
            "ImgBed": {
                "AccessKeyId": "",
                "AccessKeySecret": "",
                "BucketName": "",
                "EndPoint": "",
                "Folder": ""
            }
        }
        create_file(CONFIG_PATH, json.dumps(config_template))

create_root()

from .media import media_app
from .file import file_app

app = typer.Typer()
app.add_typer(file_app, name='file')
app.add_typer(media_app, name='media')



import os
import ffmpeg
import typer
import time
import yt_dlp
from rich import print
from typing import Annotated
from pathlib import Path
from pydub import AudioSegment
from moviepy.editor import VideoFileClip, AudioFileClip

media_app = typer.Typer(help="Media related commands")

def is_video_file(filepath: str):
    try:
        # 尝试加载文件作为视频文件
        video = VideoFileClip(filepath)
        video.close()
        return True
    except Exception as e:
        return False

def is_audio_file(filepath: str):
    try:
        # 尝试加载文件作为音频文件
        audio = AudioSegment.from_file(filepath)
        return True
    except Exception as e:
        return False

@media_app.command("2gif", help="Convert video to gif")
def to_gif(input_file: Annotated[str, typer.Option("--input-file", "-i", help="The path to your file you wang to convert to gif")], output_file: Annotated[str, typer.Option("--output-file", "-o", help="The path to the gif file")]):
    video = VideoFileClip(input_file)
    video.write_gif(output_file)
    
@media_app.command("cut", help="Cut video or audio file")
def cut(input_file: Annotated[str, typer.Option("--input-file", "-i", help="The path to the file you want to cut")], start_time: Annotated[str, typer.Option("--start-time", "-s", help="The start time of the clip (in seconds)")], end_time: Annotated[str, typer.Option("--end-time", "-e", help="The end time of the clip (in seconds)")], output_file: Annotated[str, typer.Option("--output-file", "-o",  help="The path of the cutted file")] = None):
    file_item = Path(input_file)
    if output_file is None:
        output_file = f"{file_item.stem} cutted{file_item.suffix}"
    if is_video_file(input_file):
        # 视频文件
        video = VideoFileClip(input_file)
        # 剪切视频
        video_cut = video.subclip(start_time, end_time)
        # 保存剪切后的视频
        video_cut.write_videofile(output_file, codec="libx264")
    elif is_audio_file(input_file):
        # 音频文件
        audio = AudioFileClip(input_file)
        # 剪切视频
        audio_cut = audio.subclip(start_time, end_time)
        # 保存剪切后的视频
        audio_cut.write_audiofile(output_file)

@media_app.command("convert", help="Convert video or audio file to another format")
def convert(input_file: Annotated[str, typer.Option("--input-file", "-i", help="The path to the file you want to convert")], output_file: Annotated[str, typer.Option("--output-file", "-o", help="The path to the file converted")]):
    if not os.path.exists(input_file):
        print(f"文件[bold]{input_file}[/bold]不存在")
        raise typer.Abort()
    ffmpeg.input(input_file).output(output_file).run()

@media_app.command("download", help="Download media from url")
def download(url: Annotated[str, typer.Option("--url", "-u", help="The url of the media you want to download")], format: Annotated[str, typer.Option("--format", "-f", help="The format of the media you want to get. (now only support mp4 or mp3)")]):
    download_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())
    # 定义下载选项
    ydl_opts = {
        'format': 'bestvideo+bestaudio/best',
        'outtmpl': f'{download_time} - %(title)s.%(ext)s',  # 自定义文件名和文件夹结构
    }
    if format == 'mp3':
        ydl_opts.update({
            'postprocessors': [{
                'key': 'FFmpegExtractAudio',
                'preferredcodec': 'mp3',
                'preferredquality': '0',
            }],
        })
    elif format == 'mp4':
        ydl_opts.update({
            'postprocessors': [
                {
                    'key': 'FFmpegVideoConvertor',
                    'preferedformat': 'mp4'
                },
                {
                    'key': 'EmbedThumbnail',  # 嵌入缩略图
                },
            ]
        })

    # 使用yt_dlp下载视频并提取音频
    with yt_dlp.YoutubeDL(ydl_opts) as ydl:
        ydl.download([url])


def create_file(path: str, content = ''):
    with open(path, 'w') as fp:
            # 在这里可以写入文件内容（如果需要）
            fp.write(content)
            
def read_file(path: str):
    with open(path, mode="r", encoding="utf-8") as f:
        return f.read()


import os
import typer
import oss2
import time
import pymupdf
from typing import Annotated
from tqdm import tqdm
from rich import print
from .config.config import config

file_app = typer.Typer(help="File utility")

def delete_file(path: str, temp: bool = True):
    file_name = path.split('/')[-1]
    if temp:
        os.rename(path, os.path.join(config.TRASH_PATH, file_name))
    else:
        os.remove(path=path)
    print(f"{path} 删除成功")
        
def get_top_files_in_directory(path: str):
    all_files = []  # 用于保存所有文件的全路径
    # 列出当前文件夹中的所有文件和子文件夹
    for filename in os.listdir(path):
        file_path = os.path.join(path, filename)
        if os.path.isfile(file_path):
            all_files.append(file_path)
    return all_files

def get_all_files_in_directory(path: str):
    all_files = []  # 用于保存所有文件的路径
    # 递归遍历文件夹
    for foldername, subfolders, filenames in os.walk(path):
        for filename in filenames:
            file_path = os.path.join(foldername, filename)  # 构建文件的完整路径
            all_files.append(file_path)
    return all_files
    
@file_app.command("pdf2png", help="Convert PDF to PNG")
def pdf2png(path: Annotated[str, typer.Option("--path", "-p", help="Path to the PDF file")]):
    doc = pymupdf.open(path) # open a document
    with tqdm(total=len(doc), unit='张', desc='转换中') as progress_bar:
        for page in doc: # iterate the document pages
            image = page.get_pixmap(matrix=pymupdf.Matrix(2, 2)) # get plain text encoded as UTF-8
            image.pil_save("page-%i.png" % page.number)
            progress_bar.update(1)
    print('转换完成')

@file_app.command("upload", help="Upload file to OSS")
def upload(path: Annotated[str, typer.Option("--path", "-p", help="Path to the file")]):
    auth = oss2.Auth(config.get('ImgBed').get('AccessKeyId'), config.get('ImgBed').get('AccessKeySecret'))
    bucket = oss2.Bucket(auth, config.get('ImgBed').get('EndPoint'), config.get('ImgBed').get('BucketName'))
    file_suffix = path.split('.')[-1]
    total_bytes = os.path.getsize(path)
    file_name = f"{config.get('ImgBed').get('Folder')}/{time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())}.{file_suffix}"
    # 使用tqdm包装上传过程，显示进度条
    with tqdm(total=total_bytes, unit='B', unit_scale=True, desc='上传中') as progress_bar:
        def callback_wrapper(consumed_bytes, total_bytes):
            progress_bar.update(consumed_bytes - progress_bar.n)
            return consumed_bytes
        # 执行上传
        oss2.resumable_upload(
            bucket=bucket,
            key=f"{file_name}",
            filename=path,
            progress_callback=callback_wrapper
        )
    print(f'文件上传成功，重命名为{file_name}')

@file_app.command("delete", help="Delete file")
def delete(path: Annotated[str, typer.Option("--path", "-p", help="Path to the file/folder")], include: Annotated[str, typer.Option("--include", "-i", help="Filter file name")], recursion: Annotated[bool, typer.Option("--recursion", "-r", help="Recursion to sub-directory")] = False, force: Annotated[bool, typer.Option("--force", "-f", help="Force delete")] = False):
    if recursion:
        file_path_list = get_all_files_in_directory(path=path)
    else: 
        file_path_list = get_top_files_in_directory(path=path)
    file_path_list = list(filter(lambda file_path: include in file_path, file_path_list))
    print(f"要删除的文件如下:")
    for file_path in file_path_list:
        print(f'{file_path}')
    print(f'共计 [bold red]{len(file_path_list)}[/bold red] 个')
    if len(file_path_list) == 0:
        print('程序结束')
        return
    print(f"确认删除吗?[bold red]{'注意，你已经选择-f，此项行为将会直接从磁盘删除文件不可恢复！' if force else ''}[/bold red]")
    user_ans = typer.prompt("yes(y)/no(n)", default='n')
    if user_ans == 'yes' or user_ans == 'y':
        for file_path in file_path_list:
            if force:
                delete_file(file_path, temp=False)
            else:
                delete_file(file_path)
    else:
        print('程序结束')


#!/usr/bin/env python
# -*- coding:utf-8 -*-
from setuptools import setup, find_packages

with open("./README.md", "r", encoding="utf-8") as fp:
    long_description = fp.read()

with open("./LICENSE", "r", encoding="utf-8") as fp:
    license = fp.read()
    
setup(
    name="qin",
    version="0.1.0",
    keywords=["pip", "tools"],
    description="Interesting toolset",
    entry_points={
        'console_scripts': [
            'qin = qin.main:app'
        ]
    },
    python_requires='>=3.10',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license=license,
    url="https://github.com/Alndaly/qin-cli",
    author="Kinda Hall",
    maintainer="Kinda Hall",
    maintainer_email="1142704468@qq.com",
    author_email="1142704468@qq.com",
    packages=find_packages(),
    include_package_data=True,
    platforms="macOS",
    install_requires=['typer', 'ffmpeg-python', 'oss2', 'gradio', 'yt-dlp', 'fastapi', 'moviepy']
)



import json
from .base import CONFIG_PATH
from ..common.file import read_file

def load_config():
    config_str = read_file(CONFIG_PATH)
    config = json.loads(config_str)
    return config

class Config(dict):
    def __init__(self, d=None):
        super().__init__()
        for k, v in d.items():
            self[k] = v
    
    def __getitem__(self, key):
        return super().__getitem__(key)

    def get(self, key, default=None):
        try:
            return self[key]
        except KeyError as e:
            return default
        except Exception as e:
            raise e

config = Config(load_config())
